@@ -1,5 +1,5 @@
 /*
- * Copyright (c) [2011-2013] Novell, Inc.
+ * Copyright (c) [2011-2014] Novell, Inc.
  *
  * All Rights Reserved.
  *
@@ -27,6 +27,9 @@
 #include <glob.h>
 #include <string.h>
 #include <mntent.h>
+#include <sys/acl.h>
+#include <acl/libacl.h>
+#include <set>
 #include <boost/algorithm/string.hpp>
 
 #include "snapper/Snapper.h"
@@ -50,7 +53,7 @@ namespace snapper
     ConfigInfo::ConfigInfo(const string& config_name)
 	: SysconfigFile(CONFIGSDIR "/" + config_name), config_name(config_name), subvolume("/")
     {
-	if (!getValue("SUBVOLUME", subvolume))
+	if (!getValue(KEY_SUBVOLUME, subvolume))
 	    throw InvalidConfigException();
     }
 
@@ -58,7 +61,7 @@ namespace snapper
     void
     ConfigInfo::checkKey(const string& key) const
     {
-	if (key == "SUBVOLUME" || key == "FSTYPE")
+	if (key == KEY_SUBVOLUME || key == KEY_FSTYPE)
 	    throw InvalidConfigdataException();
 
 	try
@@ -89,7 +92,7 @@ namespace snapper
 	}
 
 	string fstype = "btrfs";
-	config_info->getValue("FSTYPE", fstype);
+	config_info->getValue(KEY_FSTYPE, fstype);
 	filesystem = Filesystem::create(fstype, config_info->getSubvolume());
 
 	y2mil("subvolume:" << config_info->getSubvolume() << " filesystem:" <<
@@ -359,8 +362,8 @@ namespace snapper
 	try
 	{
 	    SysconfigFile config(CONFIGSDIR "/" + config_name);
-	    config.setValue("SUBVOLUME", subvolume);
-	    config.setValue("FSTYPE", filesystem->fstype());
+	    config.setValue(KEY_SUBVOLUME, subvolume);
+	    config.setValue(KEY_FSTYPE, filesystem->fstype());
 	}
 	catch (const FileNotFoundException& e)
 	{
@@ -368,6 +371,14 @@ namespace snapper
 	}
 
 	filesystem->createConfig();
+
+#if 1
+	if (subvolume == "/" && filesystem->fstype() == "btrfs" &&
+	    access("/usr/lib/snapper/plugins/grub", X_OK) == 0)
+	{
+	    SystemCmd cmd("/usr/lib/snapper/plugins/grub --enable");
+	}
+#endif
     }
 
 
@@ -379,6 +390,14 @@ namespace snapper
 
 	auto_ptr<Snapper> snapper(new Snapper(config_name));
 
+#if 1
+	if (snapper->subvolumeDir() == "/" && snapper->getFilesystem()->fstype() == "btrfs" &&
+	    access("/usr/lib/snapper/plugins/grub", X_OK) == 0)
+	{
+	    SystemCmd cmd("/usr/lib/snapper/plugins/grub --disable");
+	}
+#endif
+
 	Snapshots& snapshots = snapper->getSnapshots();
 	for (Snapshots::iterator it = snapshots.begin(); it != snapshots.end(); )
 	{
@@ -428,6 +447,176 @@ namespace snapper
     }
 
 
+    void
+    Snapper::setConfigInfo(const map<string, string>& raw)
+    {
+	for (map<string, string>::const_iterator it = raw.begin(); it != raw.end(); ++it)
+	    config_info->setValue(it->first, it->second);
+
+	config_info->save();
+
+	if (raw.find(KEY_ALLOW_USERS) != raw.end() || raw.find(KEY_ALLOW_GROUPS) != raw.end() ||
+	    raw.find(KEY_SYNC_ACL) != raw.end())
+	{
+	    bool sync_acl;
+	    if (config_info->getValue(KEY_SYNC_ACL, sync_acl) && sync_acl == true)
+		syncAcl();
+	}
+    }
+
+
+    void
+    Snapper::syncAcl() const
+    {
+	vector<uid_t> uids;
+	vector<string> users;
+	if (config_info->getValue(KEY_ALLOW_USERS, users))
+	{
+	    for (vector<string>::const_iterator it = users.begin(); it != users.end(); ++it)
+	    {
+		uid_t uid;
+		if (!get_user_uid(it->c_str(), uid))
+		    throw InvalidUserException();
+		uids.push_back(uid);
+	    }
+	}
+
+	vector<gid_t> gids;
+	vector<string> groups;
+	if (config_info->getValue(KEY_ALLOW_GROUPS, groups))
+	{
+	    for (vector<string>::const_iterator it = groups.begin(); it != groups.end(); ++it)
+	    {
+		gid_t gid;
+		if (!get_group_gid(it->c_str(), gid))
+		    throw InvalidGroupException();
+		gids.push_back(gid);
+	    }
+	}
+
+	syncAcl(uids, gids);
+    }
+
+
+    static void
+    set_acl_permissions(acl_entry_t entry)
+    {
+	acl_permset_t permset;
+	if (acl_get_permset(entry, &permset) != 0)
+	    throw AclException();
+
+	if (acl_add_perm(permset, ACL_READ) != 0 || acl_delete_perm(permset, ACL_WRITE) != 0 ||
+	    acl_add_perm(permset, ACL_EXECUTE) != 0)
+	    throw AclException();
+    };
+
+
+    static void
+    add_acl_permissions(acl_t* acl, acl_tag_t tag, const void* qualifier)
+    {
+	acl_entry_t entry;
+	if (acl_create_entry(acl, &entry) != 0)
+	    throw AclException();
+
+	if (acl_set_tag_type(entry, tag) != 0)
+	    throw AclException();
+
+	if (acl_set_qualifier(entry, qualifier) != 0)
+	    throw AclException();
+
+	set_acl_permissions(entry);
+    };
+
+
+    void
+    Snapper::syncAcl(const vector<uid_t>& uids, const vector<gid_t>& gids) const
+    {
+	SDir infos_dir = openInfosDir();
+
+	acl_t acl = acl_get_fd(infos_dir.fd());
+	if (!acl)
+	    throw AclException();
+
+	set<uid_t> remaining_uids = set<uid_t>(uids.begin(), uids.end());
+	set<gid_t> remaining_gids = set<gid_t>(gids.begin(), gids.end());
+
+	acl_entry_t entry;
+	if (acl_get_entry(acl, ACL_FIRST_ENTRY, &entry) == 1)
+	{
+	    do {
+
+		acl_tag_t tag;
+		if (acl_get_tag_type(entry, &tag) != 0)
+		    throw AclException();
+
+		switch (tag)
+		{
+		    case ACL_USER: {
+
+			uid_t* uid = (uid_t*) acl_get_qualifier(entry);
+			if (!uid)
+			    throw AclException();
+
+			if (contains(remaining_uids, *uid))
+			{
+			    remaining_uids.erase(*uid);
+
+			    set_acl_permissions(entry);
+			}
+			else
+			{
+			    if (acl_delete_entry(acl, entry) != 0)
+				throw AclException();
+			}
+
+		    } break;
+
+		    case ACL_GROUP: {
+
+			gid_t* gid = (gid_t*) acl_get_qualifier(entry);
+			if (!gid)
+			    throw AclException();
+
+			if (contains(remaining_gids, *gid))
+			{
+			    remaining_gids.erase(*gid);
+
+			    set_acl_permissions(entry);
+			}
+			else
+			{
+			    if (acl_delete_entry(acl, entry) != 0)
+				throw AclException();
+			}
+
+		    } break;
+
+		}
+
+	    } while (acl_get_entry(acl, ACL_NEXT_ENTRY, &entry) == 1);
+	}
+
+	for (set<uid_t>::const_iterator it = remaining_uids.begin(); it != remaining_uids.end(); ++it)
+	{
+	    add_acl_permissions(&acl, ACL_USER, &(*it));
+	}
+
+	for (set<gid_t>::const_iterator it = remaining_gids.begin(); it != remaining_gids.end(); ++it)
+	{
+	    add_acl_permissions(&acl, ACL_GROUP, &(*it));
+	}
+
+	if (acl_calc_mask(&acl) != 0)
+	    throw AclException();
+
+	if (acl_set_fd(infos_dir.fd(), acl) != 0)
+	    throw AclException();
+
+	if (acl_free(acl) != 0)
+	    throw AclException();
+    }
+
+
     static bool
     is_subpath(const string& a, const string& b)
     {
